﻿using System;
using Alachisoft.NCache.Web.Caching;
using Alachisoft.NCache.sample.data;

// ===============================================================================
// Alachisoft (R) NCache Sample Code.
// NCache Cache Item Versioning sample
// ===============================================================================
// Copyright © Alachisoft.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
// ===============================================================================

namespace CacheItemVersioning
{
	public class CacheItemVersioning
	{
		public static void Main(string[] args)
		{
			try
			{
				//Initialize cache
				Cache cache;
				cache = NCache.InitializeCache("mycache");
				cache.Clear();

				//Cache item versioning makes possible the concurrency checks on cacheitems.
				//Scenario:
				//User X and Y has fetched data from cache
				//User Y makes changes to some cache items and write back the changes to cache
				//Now the version of cache items with User X are obsolete and it can determine 
				//so by using cache item versioning.

				Customer customer = new Customer();
				customer.Name = "Scott Prince";
				customer.Address = "95-A Barnes Road Wallingford, CT";
				customer.Gender = "Male";
				customer.ContactNo = "25632-5646";
				customer.Age = 23;

				CacheItemVersion version1 = cache.Add("Customer:ScottPrince", customer);

				//updaing the customer object in cache;
				customer.Age = 33;
				CacheItemVersion version2 = cache.Insert("Customer:ScottPrince", customer);

				if (version1.Version != version2.Version)
				{
					Console.WriteLine("Item has changed since last time it was fetched.");
					Console.WriteLine();
				}

				//GetIfNewer
				//Retrives item from cache based on CacheItemVersion
				//Get item only is version superior to version2
				Customer customer2 = (Customer) cache.GetIfNewer("Customer:ScottPrince", ref version2); //Version to be compared from cache

				if (customer2 == null)
				{
					Console.WriteLine("Latest version of item is already available");
				}
				else
				{
					Console.WriteLine("Current Version of Customer:ScottPrince is: " + version2.Version);
					PrintCustomerDetails(customer2);
				}

				//Remove
				customer.Name = "Mr. Scott n Price";
				CacheItemVersion version3 = cache.Insert("Customer:ScottPrince", customer);

				Customer customerRemoved = (Customer) cache.Remove("Customer:ScottPrince", version3);

				if (customerRemoved == null)
				{
					Console.WriteLine("Remove failed. The newer version of item exists in cache.");
				}
				else
				{
					Console.WriteLine("Following Customer is removed from cache:");
					PrintCustomerDetails(customerRemoved);
				}

				//Always manipulate the latest item as follows.

				CacheItemVersion version4 = cache.Insert("Customer:ScottPrince", customer);
				customer2 = (Customer) cache.GetIfNewer("Customer:ScottPrince", ref version4);

			    Console.WriteLine(customer2 == null ? "Item version available is latest!" : "Latest available Item version is fetched from cache");

			    //Must dispose cache
				cache.Dispose();

				Environment.Exit(0);

			}
			catch (Exception ex)
			{
				Console.WriteLine(ex.Message);
				Environment.Exit(0);
			}
		}

        /// <summary>
        /// Method for printing details of customer type.
        /// </summary>
        /// <param name="customer"></param>
		 private static void PrintCustomerDetails(Customer customer)
		 {
			Console.WriteLine();
			Console.WriteLine("Customer Details are as follows: ");
			Console.WriteLine("Name: " + customer.Name);
			Console.WriteLine("Age: " + customer.Age);
			Console.WriteLine("Gender: " + customer.Gender);
			Console.WriteLine("Contact No: " + customer.ContactNo);
			Console.WriteLine("Address: " + customer.Address);
		 }
	}

}